package de.tudarmstadt.informatik.hostage.ui2.fragment;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLES20;
import android.opengl.GLSurfaceView.Renderer;
import android.opengl.Matrix;

public class HomeGLRenderer implements Renderer {
	public static int loadShader(int type, String source) {
		int shader = GLES20.glCreateShader(type);
		GLES20.glShaderSource(shader, source);
		GLES20.glCompileShader(shader);
		return shader;
	}
	
	public static int loadProgram(String vertexSource, String fragmentSource) {
		int program = GLES20.glCreateProgram();
		GLES20.glAttachShader(program, loadShader(GLES20.GL_VERTEX_SHADER, vertexSource));
		GLES20.glAttachShader(program, loadShader(GLES20.GL_FRAGMENT_SHADER, fragmentSource));
		GLES20.glLinkProgram(program);
		return program;
	}
	
	private int program;
	private FloatBuffer vertexBuffer;
	private float [] mvp;

	public HomeGLRenderer() {}
	
	public void onSurfaceCreated(GL10 arg0, EGLConfig arg1) {
		GLES20.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

		final float [] positions = {
				0.0f, 0.5f, 0.0f, 1.0f,
				0.5f, -0.5f, 0.0f, 1.0f,
				-0.5f, -0.5f, 0.0f, 1.0f,
		};
		ByteBuffer bb = ByteBuffer.allocateDirect(4 * positions.length);
		bb.order(ByteOrder.nativeOrder());
		vertexBuffer = bb.asFloatBuffer();
		vertexBuffer.put(positions);
		vertexBuffer.position(0);
		
		mvp = new float[16];
		Matrix.setIdentityM(mvp, 0);
		
		final String vertexSource =
				"uniform mat4 mvp;" +
				"attribute vec4 position;" +
				"void main() {" +
				"	gl_Position = mvp * position;" +
				"}";
		final String fragmentSource =
				"precision mediump float;" +
				"uniform vec4 color;" +
				"void main() {" +
				"	gl_FragColor = color;" +
				"}";
		program = loadProgram(vertexSource, fragmentSource);
	}
	
	public void onDrawFrame(GL10 arg0) {
		GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
		GLES20.glUseProgram(program);
		int positionAttribLoc = GLES20.glGetAttribLocation(program, "position");
		int colorUniformLoc = GLES20.glGetUniformLocation(program, "color");
		int mvpUniformLoc = GLES20.glGetUniformLocation(program, "mvp");
		float [] color = {0.0f, 1.0f, 0.0f, 1.0f};
		GLES20.glUniform4fv(colorUniformLoc, 1, color, 0);
		Matrix.rotateM(mvp, 0, 4.0f, 0.0f, 0.0f, 1.0f);
		GLES20.glUniformMatrix4fv(mvpUniformLoc, 1, false, mvp, 0);
		GLES20.glEnableVertexAttribArray(positionAttribLoc);
		GLES20.glVertexAttribPointer(positionAttribLoc, 4, GLES20.GL_FLOAT, false, 0, vertexBuffer);
		GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 3);
		GLES20.glDisableVertexAttribArray(positionAttribLoc);
	}

	public void onSurfaceChanged(GL10 arg0, int width, int height) {
		GLES20.glViewport(0, 0, width, height);
	}
}
